% Please cite the paper and co-authors accompanying this code when using it for
% replication or any other purposes. Copyright Anton Velinov.
function [Y,K,T,Zbar,Xbar,bold_y,z] = initialization(y,p,M,intercept,slope)
% This code defines the exogenous matrices and values to be used
% throughout.

% INPUTS:
% y - (Kx(T+p)) matrix of endogenous variables
    % K: number endogenous variables (output)
    % T: number of observations not taken as given (output)
% p - number of lags
% M - number of Markov states
% intercept: intercept = 'y' for a switching intercept term, intercept = 'n' otherwise.
% slope: slope = 'y' for a switching AR matrices, slope = 'n' otherwise.

% OUTPUTS:
% Y - (KxT) matrix of endogenous variables
% K - number of endogenous variables
% T - number of observations  not taken as given
% Zbar - (TxK*p) matrix of right hand side (RHS) exogenous variables
% Xbar - matrix containing the right hand side (RHS) exogenous terms. Dimension depends on user specification.
% bold_y - (T*Kx1) vectorization of Y
% z: number of parameters to be estimated on the (RHS) of the equation.

Y = y(:,p+1:end);
[K,T] = size(Y);
bold_y = reshape(Y,T*K,1);
Zbar = zeros(T,K*p);
for i = 1:p
    Zbar(:,K*(i-1)+1:K*i) = y(:,p+1-i:T+p-i)';
end

if slope == 'n'
    if intercept == 'n' % MSH(M)-SVAR(p) Model
        Xbar = [ones(T,1) Zbar];
        z = K*(K*p + 1);
    else % MSIH(M)-SVAR(p) Model
        I_M = eye(M);
        Xbar = zeros(T,M+K*p,M);
        for m = 1:M
            Xbar(:,:,m) = [kron(ones(T,1),I_M(:,m)') Zbar];
        end
        z = K*(K*p + M);
    end
else
    if intercept == 'n' % MSAH(M)-SVAR(p) Model
        I_M = eye(M);
        Xbar = zeros(T,1+M*K*p,M);
        for m = 1:M
            Xbar(:,:,m) = [ones(T,1) kron(I_M(:,m)',Zbar)];
        end
        z = K*(M*K*p + 1);
    else % MSIAH(M)-SVAR(p) Model
        Xbar = [ones(T,1) Zbar];
        z = K*(M*K*p + M);
    end
end

end